function grad = grad_compute(A,V,B,W,y,x,V0)

% returns the gradient of the negative log-likelihood at point x

% Inputs:
% A:  dynamics matrix d x d
% V:  dynamics covariance d x d
% B:  measurement matrices (cell of length T, each matrix has size b_t x d)
% W:  measurement covariance (cell of length T, each matrix b_t x b_t)
% y:  observation vector (cell of length T, each matrix has size b_t x 1)
% theta:    threshold
% x:  point of gradient computation
% V0: covariance of the initial state (default: V0 = V)

% Written by Eftychios A. Pnevmatikakis, eftychios@stat.columbia.edu
% August 2012

d = size(A,1);
T = length(B);
Vinv = V\speye(d);

if isvector(x)
    x = reshape(x,d,T);
end

if nargin < 7
    V0 = V;
    V0inv = Vinv;
else
    V0inv = V0\speye(d);
end

Em = A'*Vinv;

%%%%% gradient computing function

% computes the gradient of the negative log-likelihood function at point x

grad = zeros(d*T,1);

for t = 1:T
    grad((t-1)*d + (1:d)) = - B{t}'*(W{t}\(y{t}-B{t}*x(:,t)));
end

if any(x(:))
    grad(1:d) = grad(1:d) + V0inv*x(:,1) - Em*(x(:,2)-A*x(:,1));
    for t = 2:T-1
        grad((t-1)*d + (1:d)) = grad((t-1)*d + (1:d)) - Em*(x(:,t+1)-A*x(:,t)) + Vinv*(x(:,t)-A*x(:,t-1));
    end
    grad((T-1)*d + (1:d)) = grad((T-1)*d + (1:d)) + Vinv*(x(:,T)-A*x(:,T-1));
end